/**
 ** Script de Florent TOUCHARD (JackDesBwa)
 ** http://mozaique.desbwa.org/
 ** Version 0.2
 */

/***********************************************************************

  Copyright (c) 2010 Florent TOUCHARD (JackDesBwa)

  Permission is hereby granted, free of charge, to any person obtaining
  a copy of this software and associated documentation files
  (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

***********************************************************************/

/** Inclusions **/
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glob.h>
#include <wand/MagickWand.h>

/** Paramètres **/
#define FOND "/home/jackdesbwa/fond.jpg"
#define GLOB "/mnt/donnees/Photos/*/*\\.[jJ][pP][gG]"
#define MARGE -10
#define SECONDES 120
#define NB_LOTS -1
#define NB_PAR_LOT 10
#define WIDTH_MIN 150
#define WIDTH_MAX 200

/** Globales **/
MagickWand * mw_background = NULL;
MagickWand * mw_mini = NULL;
DrawingWand * drawing_wand = NULL;
PixelWand * pixel_wand = NULL;
glob_t globbuf;
char run = 1;

/** Initialisation **/
void genesis() {
	MagickWandGenesis();
	mw_background = NewMagickWand();
	drawing_wand = NewDrawingWand();
	pixel_wand = NewPixelWand();
	srand(time(NULL));
	glob(GLOB, GLOB_NOCHECK, NULL, &globbuf);
}

/** Choix aléatoire de la photo **/
char * choix_photo() {
	int random_val = rand() % globbuf.gl_pathc;
	return globbuf.gl_pathv[random_val];
}

/** Orientation **/
void orientation() {
	PixelSetColor(pixel_wand, "none");
	// Rotations
	switch(MagickGetImageOrientation(mw_mini)) {
		case 3 :
		case 4 :
			MagickRotateImage(mw_mini, pixel_wand, 180);
			break;
		case 5 :
		case 6 :
			MagickRotateImage(mw_mini, pixel_wand, 90);
			break;
		case 7 :
		case 8 :
			MagickRotateImage(mw_mini, pixel_wand, -90);
			break;
	}
	// Reflexions
	switch(MagickGetImageOrientation(mw_mini)) {
		case 2 :
		case 3 :
		case 5 :
		case 7 :
			MagickFlopImage(mw_mini);
			break;
	}
	MagickSetImageOrientation(mw_mini, 1);
}

/** Mise à l'échelle **/
void scale() {
	int random_val = (rand() % (WIDTH_MAX - WIDTH_MIN)) + WIDTH_MIN;
	unsigned long int width = MagickGetImageWidth(mw_mini);
	unsigned long int height = MagickGetImageHeight(mw_mini);
	if (width > height) {
		height = height * random_val / width;
		width = random_val;
	} else {
		width = width * random_val / height;
		height = random_val;
	}
	PixelSetColor(pixel_wand, "none");
	MagickSetImageBackgroundColor(mw_mini, pixel_wand);
	MagickScaleImage(mw_mini, width, height);
}

/** Effet polaroïd **/
void polaroid() {
	int random_val = (rand() % 30) - 15;
	MagickWand * mw_image;

	// Bordure
	PixelSetColor(pixel_wand, "white");
	MagickBorderImage(mw_mini, pixel_wand, 3, 3);
	
	// Ondulation
	PixelSetColor(pixel_wand, "none");
	MagickSetImageBackgroundColor(mw_mini, pixel_wand);
	MagickWaveImage(
		mw_mini,
		2*MagickGetImageWidth(mw_mini)/100,
		2*MagickGetImageHeight(mw_mini)
	);

	// Ombres
	mw_image = CloneMagickWand(mw_mini);
	PixelSetColor(pixel_wand, "black");
	MagickSetImageBackgroundColor(mw_mini, pixel_wand);
	MagickShadowImage(mw_mini, 100, 8, 4, 4);
	MagickFlopImage(mw_mini);
	MagickCompositeImage(mw_mini, mw_image, OverCompositeOp, 8, 8);
	DestroyMagickWand(mw_image);
	mw_image = CloneMagickWand(mw_mini);
	MagickShadowImage(mw_mini, 50, 20, -20, -20);
	MagickCompositeImage(mw_mini, mw_image, OverCompositeOp, 20, 20);
	DestroyMagickWand(mw_image);

	// Angle
	PixelSetColor(pixel_wand, "none");
	MagickRotateImage(mw_mini, pixel_wand, random_val);
}

void fusion() {
	unsigned long int posx = rand() % (MagickGetImageWidth(mw_background) - MagickGetImageWidth(mw_mini) - 2*MARGE) + MARGE;
	unsigned long int posy = rand() % (MagickGetImageHeight(mw_background) - MagickGetImageHeight(mw_mini) - 2*MARGE) + MARGE;
	PixelSetColor(pixel_wand, "none");
	MagickCompositeImage(mw_background, mw_mini, SrcOverCompositeOp, posx, posy);
}

/** Terminaison **/
void terminus() {
	globfree(&globbuf);
	if (pixel_wand) DestroyPixelWand(pixel_wand);
	if (drawing_wand) DestroyDrawingWand(drawing_wand);
	if (mw_mini) DestroyMagickWand(mw_mini);
	if (mw_background) DestroyMagickWand(mw_background);
	MagickWandTerminus();
}

/** Interruption **/
void interrupt() {
	puts("Extinction demandée, termine le traitement en cours...");
	run = 0;
}

/** Point d'entrée **/
int main(int argc,char **argv) {
	int lots;
	/* Initialisation */
	genesis();
	signal(SIGINT, interrupt);
	signal(SIGTERM, interrupt);
	MagickReadImage(mw_background, FOND);

#if NB_LOTS > 0
	for(lots=0; lots<NB_LOTS && run; lots++) {
#else
	while(run) {
#endif
		int i;
		for(i=0; i<NB_PAR_LOT; i++) {
			/* Choix aléatoire de la photo */
			mw_mini = NewMagickWand();
			MagickReadImage(mw_mini, choix_photo());

			/* Traitement de la miniature */
			orientation();
			scale();
			polaroid();
			fusion();
			
			/* Fin de la miniature*/
			DestroyMagickWand(mw_mini);
			mw_mini = NULL;
		}

		/* Enregistrement */
		MagickWriteImage(mw_background, FOND);
		
#if SECONDES > 0
		sleep(SECONDES);
#endif
	}
	terminus();
	return 0;
}
